/*
 * Machine dependent access functions for RTC registers.
 */
#ifndef _M68KNOMMU_MC146818RTC_H
#define _M68KNOMMU_MC146818RTC_H

/* empty include file to satisfy the include in genrtc.c/ide-geometry.c */

#endif /* _M68KNOMMU_MC146818RTC_H */
