/* csum_copy_to_user.S: Checksum+copy to userspace.
 *
 * Copyright (C) 2005 David S. Miller (davem@davemloft.net)
 */

#define EX_ST(x)		\
98:	x;			\
	.section .fixup;	\
	.align 4;		\
99:	retl;			\
	 mov	-1, %o0;	\
	.section __ex_table,"a";\
	.align 4;		\
	.word 98b, 99b;		\
	.text;			\
	.align 4;

#define FUNC_NAME		__csum_partial_copy_to_user
#define STORE(type,src,addr)	type##a src, [addr] %asi

#include "csum_copy.S"
