/*
 * Copyright (C) 2005 Meilhaus Electronic GmbH (support@meilhaus.de)
 *
 * Source File : meids.h
 * Author      : GG (Guenter Gebhardt)  <g.gebhardt@meilhaus.de>
 */

#ifndef _MEIDS_H_
#define _MEIDS_H_

#ifdef __KERNEL__

/*=============================================================================
  Driver names
  ===========================================================================*/

#define MEMAIN_NAME				"memain"
#define ME1000_NAME				"me1000"
#define ME1400_NAME				"me1400"
#define ME1600_NAME				"me1600"
#define ME4600_NAME				"me4600"
#define ME6000_NAME				"me6000"
#define ME0600_NAME				"me0600"	//"me630"
#define ME8100_NAME				"me8100"
#define ME8200_NAME				"me8200"
#define ME0900_NAME				"me0900"	//"me9x"
//#define MEPHISTO_S1_NAME                      "mephisto_s1"
#define MEDUMMY_NAME			"medummy"

#endif
#endif
