/*
** asm/setup.h -- Definition of the Linux/bfin setup information
**
** This file is subject to the terms and conditions of the GNU General Public
** License.  See the file COPYING in the main directory of this archive
** for more details.
**
** Copyright Lineo, Inc 2001          Tony Kou
**
*/

#ifndef _BFIN_SETUP_H
#define _BFIN_SETUP_H

#define COMMAND_LINE_SIZE	512

#endif				/* _BFIN_SETUP_H */
